import { defineStore } from 'pinia'
import { turtleSoupService } from '@/services/games/turtlesoup'
import {
  type TurtleSoupGameHostView,
  type TurtleSoupGamePlayerView,
  type TurtleSoupTheme,
  type TurtleSoupPuzzle,
  type TurtleSoupGameSettings,
  TurtleSoupGameStatus,
  TurtleSoupDifficulty
} from '@/types/games/turtlesoup'
import { RoomStatus } from '@/types/room'
import { ref, computed } from 'vue'

export const useTurtleSoupStore = defineStore('turtlesoup', () => {
  // 状态
  const hostView = ref<TurtleSoupGameHostView | null>(null)
  const playerView = ref<TurtleSoupGamePlayerView | null>(null)
  const loading = ref(false)
  const submitting = ref(false)
  const error = ref<string | null>(null)
  const isHost = ref(false)
  const localDraft = ref('')

  // 游戏设置
  const gameSettings = ref<TurtleSoupGameSettings>({
    themeId: '',
    puzzleId: '',
    difficulty: TurtleSoupDifficulty.MEDIUM,
    maxPlayers: 10
  })

  // 主题和题目
  const availableThemes = ref<TurtleSoupTheme[]>([])
  const availablePuzzles = ref<TurtleSoupPuzzle[]>([])

  // 计算属性
  const currentView = computed(() => isHost.value ? hostView.value : playerView.value)
  const gameStatus = computed(() => currentView.value?.status || TurtleSoupGameStatus.CREATED)
  const isGameActive = computed(() => gameStatus.value === TurtleSoupGameStatus.ACTIVE)
  const isGameEnded = computed(() => gameStatus.value === TurtleSoupGameStatus.COMPLETED || gameStatus.value === TurtleSoupGameStatus.ABANDONED)

  // 游戏时长
  const gameDuration = computed(() => {
    const view = currentView.value
    if (!view || !view.startTime) return 0
    return Math.floor(((view.currentTime || Date.now()) - view.startTime) / 60000)
  })

  // 游戏进度
  const gameProgress = computed(() => currentView.value?.progress || 0)

  // 房间代码
  const roomCode = computed(() => (isHost.value && hostView.value ? hostView.value.roomId : ''))

  // 重置
  function resetState() {
    hostView.value = null
    playerView.value = null
    loading.value = false
    submitting.value = false
    error.value = null
    isHost.value = false
    localDraft.value = ''
    gameSettings.value = {
      themeId: '',
      puzzleId: '',
      difficulty: TurtleSoupDifficulty.MEDIUM,
      maxPlayers: 10
    }
  }

  function setViewType(role: 'host' | 'player') {
    isHost.value = role === 'host'
  }

  // 主题
  async function loadThemes() {
    if (availableThemes.value.length > 0) return availableThemes.value
    loading.value = true
    error.value = null
    try {
      const result = await turtleSoupService.getThemes()
      if (result.success && result.data) {
        availableThemes.value = result.data
        return result.data
      } else {
        error.value = result.message || '加载主题失败'
        return []
      }
    } catch (e) {
      error.value = e instanceof Error ? e.message : '未知错误'
      return []
    } finally {
      loading.value = false
    }
  }

  async function unlockTheme(themeId: string) {
    const result = await turtleSoupService.unlockTheme(themeId)
    if (result.success) return result
    return null
  }

  // 题目
  async function loadPuzzles(themeId: string, difficulty?: TurtleSoupDifficulty) {
    loading.value = true
    error.value = null
    try {
      const result = await turtleSoupService.getPuzzles(themeId, difficulty)
      if (result.success && result.data) {
        availablePuzzles.value = result.data
        return result.data
      } else {
        error.value = result.message || '加载题目失败'
        return []
      }
    } catch (e) {
      error.value = e instanceof Error ? e.message : '未知错误'
      return []
    } finally {
      loading.value = false
    }
  }

  // 游戏主流程
  async function createGame(settings: TurtleSoupGameSettings) {
    submitting.value = true
    error.value = null
    try {
      gameSettings.value = settings
      const result = await turtleSoupService.createGame(settings)
      if (result.success && result.data) {
        return {
          success: true,
          gameId: result.data.gameId,
          roomId: result.data.roomId
        }
      } else {
        error.value = result.message || '创建游戏失败'
        return { success: false, message: error.value }
      }
    } catch (e) {
      error.value = e instanceof Error ? e.message : '创建游戏时发生错误'
      return { success: false, message: error.value }
    } finally {
      submitting.value = false
    }
  }

  // 加载房间
  async function loadHostGame(gameId: string) {
    loading.value = true
    error.value = null
    isHost.value = true
    try {
      const result = await turtleSoupService.getHostGameData(gameId)
      if (result.success && result.data) {
        hostView.value = result.data
        return true
      } else {
        error.value = result.message || '加载游戏数据失败'
        return false
      }
    } catch (e) {
      error.value = e instanceof Error ? e.message : '未知错误'
      return false
    } finally {
      loading.value = false
    }
  }

  async function loadPlayerGame(gameId: string) {
    loading.value = true
    error.value = null
    isHost.value = false
    try {
      const result = await turtleSoupService.getPlayerGameData(gameId)
      if (result.success && result.data) {
        playerView.value = result.data
        return true
      } else {
        error.value = result.message || '加载游戏数据失败'
        return false
      }
    } catch (e) {
      error.value = e instanceof Error ? e.message : '未知错误'
      return false
    } finally {
      loading.value = false
    }
  }

  function gameStatusToRoomStatus(status: TurtleSoupGameStatus): RoomStatus {
    switch (status) {
      case TurtleSoupGameStatus.CREATED:
      case TurtleSoupGameStatus.WAITING:
        return RoomStatus.WAITING
      case TurtleSoupGameStatus.ACTIVE:
        return RoomStatus.PLAYING
      case TurtleSoupGameStatus.COMPLETED:
      case TurtleSoupGameStatus.ABANDONED:
        return RoomStatus.ENDED
      default:
        return RoomStatus.WAITING
    }
  }

  async function startGame() {
    if (!isHost.value || !hostView.value) {
      error.value = '主持人视图未加载'
      return false
    }
    submitting.value = true
    error.value = null
    try {
      const result = await turtleSoupService.startGame(hostView.value.roomId)
      if (result.success) {
        hostView.value = {
          ...hostView.value,
          status: TurtleSoupGameStatus.ACTIVE,
          startTime: Date.now(),
          currentTime: Date.now()
        }
        return true
      } else {
        error.value = result.message || '开始游戏失败'
        return false
      }
    } catch (e) {
      error.value = e instanceof Error ? e.message : '开始游戏时发生错误'
      return false
    } finally {
      submitting.value = false
    }
  }

  // 更新进度
  async function updateProgress(progress: number) {
    if (!isHost.value || !hostView.value) {
      error.value = '主持人视图未加载'
      return false
    }
    submitting.value = true
    try {
      const result = await turtleSoupService.updateProgress(hostView.value.roomId, progress)
      if (result.success) {
        hostView.value = {
          ...hostView.value,
          progress
        }
        return true
      } else {
        error.value = result.message || '更新游戏进度失败'
        return false
      }
    } catch (e) {
      error.value = e instanceof Error ? e.message : '更新进度时发生错误'
      return false
    } finally {
      submitting.value = false
    }
  }

  // 刷新数据
  async function refreshGameData(gameId: string) {
    return isHost.value ? loadHostGame(gameId) : loadPlayerGame(gameId)
  }

  // 加载游戏信息 
  async function loadGameInfo(gameId: string, roomId: string) {
    try {
      error.value = null
      loading.value = true
      
      // 检查当前角色
      const role = await turtleSoupService.checkGameRole(gameId)
      
      if (role.success && role.data) {
        setViewType(role.data.isHost ? 'host' : 'player')
        
        // 根据角色加载相应的游戏视图
        const loadResult = await refreshGameData(gameId)
        
        if (loadResult) {
          return {
            success: true,
            revealedClues: role.data.revealedClues || []
          }
        } else {
          return { success: false, message: error.value || '加载游戏信息失败' }
        }
      } else {
        return { success: false, message: role.message || '检查游戏角色失败' }
      }
    } catch (e) {
      error.value = e instanceof Error ? e.message : '加载游戏信息时发生错误'
      return { success: false, message: error.value }
    } finally {
      loading.value = false
    }
  }

  // 主持人显示线索
  async function revealClue(gameId: string, roomId: string, clueIndex: number) {
    if (!isHost.value || !hostView.value) {
      return { success: false, message: '只有主持人可以显示线索' }
    }
    
    submitting.value = true
    error.value = null
    
    try {
      const result = await turtleSoupService.revealClue(hostView.value.gameId, clueIndex)
      
      if (result.success) {
        // 如果成功，刷新游戏数据
        await refreshGameData(gameId)
        return { success: true }
      } else {
        error.value = result.message || '显示线索失败'
        return { success: false, message: error.value }
      }
    } catch (e) {
      error.value = e instanceof Error ? e.message : '显示线索时发生错误'
      return { success: false, message: error.value }
    } finally {
      submitting.value = false
    }
  }

  // 主持人更新游戏设置
  async function updateGameSettings(gameId: string, roomId: string, settings: Partial<TurtleSoupGameSettings>) {
    if (!isHost.value || !hostView.value) {
      return { success: false, message: '只有主持人可以更新游戏设置' }
    }
    
    submitting.value = true
    error.value = null
    
    try {
      const result = await turtleSoupService.updateGameSettings(
        hostView.value.gameId, 
        { ...gameSettings.value, ...settings }
      )
      
      if (result.success) {
        // 更新本地游戏设置
        gameSettings.value = { ...gameSettings.value, ...settings }
        // 刷新游戏数据
        await refreshGameData(gameId)
        return { success: true }
      } else {
        error.value = result.message || '更新游戏设置失败'
        return { success: false, message: error.value }
      }
    } catch (e) {
      error.value = e instanceof Error ? e.message : '更新游戏设置时发生错误'
      return { success: false, message: error.value }
    } finally {
      submitting.value = false
    }
  }

  // 获取主题支付参数
  async function getThemePaymentParams(themeId: string) {
    if (!themeId) {
      error.value = '主题ID不能为空'
      return null
    }
    
    error.value = null
    loading.value = true
    
    try {
      // 调用服务获取支付参数
      const result = await turtleSoupService.getThemePaymentParams(themeId)
      
      if (result.success && result.data) {
        return result.data
      } else {
        error.value = result.message || '获取支付参数失败'
        return null
      }
    } catch (e) {
      error.value = e instanceof Error ? e.message : '获取支付参数时发生错误'
      return null
    } finally {
      loading.value = false
    }
  }

  return {
    // 状态
    hostView,
    playerView,
    loading,
    submitting,
    error,
    isHost,
    localDraft,
    gameSettings,
    availableThemes,
    availablePuzzles,

    // 计算属性
    currentView,
    gameStatus,
    isGameActive,
    isGameEnded,
    gameDuration,
    gameProgress,
    roomCode,

    // 操作方法
    resetState,
    setViewType,
    loadThemes,
    unlockTheme,
    loadPuzzles,
    createGame,
    loadHostGame,
    loadPlayerGame,
    gameStatusToRoomStatus,
    startGame,
    updateProgress,
    refreshGameData,
    loadGameInfo,
    revealClue,
    updateGameSettings,
    getThemePaymentParams
  }
})